﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.Devices.Enumeration;
using Windows.Devices.PointOfService;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using System.Runtime.InteropServices.WindowsRuntime;

delegate void ProcessElementCallback(KeyValuePair<string, string> kvp);

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace BarcodeScanningApp
{
    // Main Page. This is a single page app.
    public sealed partial class MainPage : Page
    {
        public static MainPage Current;

        private bool triggering = false;

        // Available symbologies for the app. More symbologies can be added to this list.
        uint[] displayedSymbologiesList = new uint[]
        {
            BarcodeSymbologies.Upca,
            BarcodeSymbologies.Ean13,
            BarcodeSymbologies.Code128,
            BarcodeSymbologies.Code39,
            BarcodeSymbologies.DataMatrix,
            BarcodeSymbologies.Qr
        };

        // Profile header. This is the standard header for a profile file.
        string[] profileHeader = new string[]
        {
            "<?xml version=\"1.0\"?>",
            "",
            "<ConfigDoc flags=\"000\" name=\"Data Collection Profiles\">",
            "<HHPReserved>",
            "<Key name=\"EXMVersion\">1.0.1</Key>",
            "<Key name=\"ContentVersion\">1.0.0</Key>",
            "</HHPReserved>"
        };

        // UPCA/EAN13 Addenda Enable/Disable sections.
        //  Enable_UPCA_EAN13_Addenda enables UPCA and EAN13 symbologies with 2 character and 5 character addendums.
        //  Disable_UPCA_EAN13_Addenda enables UPCA and EAN13 symbologies and disables 2 character and 5 character addendums for both.
        string[] UPCAEAN13AddendaSection = new string[]
        {
            "<Section flags=\"000\" name=\"Enable_UPCA_EAN13_Addenda\" id=\"UPCAEAN13AddendaEnable\">",       // Profile name specfied in this line.
            "<Key cmd=\"DEVICE\" list=\"Internal,USB\" name=\"Device Type\">Internal</Key>",                  // Profile will apply to the Internal scanner.
            "<Key cmd=\"TYPE\"  list=\"Incremental,Full\" name=\"ProfileType\">Full</Key>",                   // Profile will set default configuration before applying profile.
            "<Key cmd=\"APPLY\" list=\"true,false\" min=\"\" name=\"ApplyProfileOnLoad\">false</Key>",        // POS application controls when profile is applied.
            "",
            "<Key cmd=\"DEC_UPCA_ENABLE\" list=\"true,false\" name=\"UPC-A Enable Symbology\" id=\"Enable\" gr=\"flag\">true</Key>",                                  // Enable UPCA
            "<Key cmd=\"DEC_UPCA_CHECK_DIGIT_TRANSMIT\" list=\"true,false\" name=\"UPCA Check Digit Transmit\" id=\"Enable\" gr=\"true\">true</Key>",                 // Enable UPCA check digit transmit
            "<Key cmd=\"DEC_UPCA_2CHAR_ADDENDA_ENABLED\" list=\"true,false\" name=\"Enable 2 Digit Addenda (UPC-A)\" id=\"Addenda2Digit\" gr=\"flag\">true</Key>",    // Enable UPCA 2 char addenda
            "<Key cmd=\"DEC_UPCA_5CHAR_ADDENDA_ENABLED\" list=\"true,false\" name=\"Enable 5 Digit Addenda (UPC-A)\" id=\"Addenda5Digit\" gr=\"flag\">true</Key>",    // Enable UPCA 5 char addenda
            "",
            "<Key cmd=\"DEC_EAN13_ENABLED\" list=\"true,false\" name=\"EAN-13 Enable Symbology\" id=\"Enable\" gr=\"flag\">true</Key>",                                // Enable EAN13
            "<Key cmd=\"DEC_EAN13_CHECK_DIGIT_TRANSMIT\" list=\"true,false\" name=\"EAN13 Check Digit Transmit\" id=\"Enable\" gr=\"true\">true</Key>",                // Enable EAN13 check digit transmit
            "<Key cmd=\"DEC_EAN13_2CHAR_ADDENDA_ENABLED\" list=\"true,false\" name=\"Enable 2 Digit Addenda (EAN-13)\" id=\"Addenda2Digit\" gr=\"flag\">true</Key>",   // Enable EAN13 2 char addenda   
            "<Key cmd=\"DEC_EAN13_5CHAR_ADDENDA_ENABLED\" list=\"true,false\" name=\"Enable 5 Digit Addenda (EAN-13)\" id=\"Addenda5Digit\" gr=\"flag\">true</Key>",   // Enable EAN13 5 char addenda
            "</Section>",
            "",
            "<Section flags=\"000\" name=\"Disable_UPCA_EAN13_Addenda\" id=\"UPCAEAN13AddendaDisable\">",     // Profile name specfied in this line.
            "<Key cmd=\"DEVICE\" list=\"Internal,USB\" name=\"Device Type\">Internal</Key>",                  // Profile will apply to the Internal scanner.
            "<Key cmd=\"TYPE\"  list=\"Incremental,Full\" name=\"ProfileType\">Full</Key>",                   // Profile will set default configuration before applying profile.
            "<Key cmd=\"APPLY\" list=\"true,false\" min=\"\" name=\"ApplyProfileOnLoad\">false</Key>",        // POS application controls when profile is applied.
            "",
            "<Key cmd=\"DEC_UPCA_ENABLE\" list=\"true,false\" name=\"UPC-A Enable Symbology\" id=\"Enable\" gr=\"flag\">true</Key>",                                   // Enable UPCA
            "<Key cmd=\"DEC_UPCA_CHECK_DIGIT_TRANSMIT\" list=\"true,false\" name=\"UPCA Check Digit Transmit\" id=\"Enable\" gr=\"true\">true</Key>",                  // Enable UPCA check digit transmit
            "<Key cmd=\"DEC_UPCA_2CHAR_ADDENDA_ENABLED\" list=\"true,false\" name=\"Enable 2 Digit Addenda (UPC-A)\" id=\"Addenda2Digit\" gr=\"flag\">false</Key>",    // Disable UPCA 2 char addenda
            "<Key cmd=\"DEC_UPCA_5CHAR_ADDENDA_ENABLED\" list=\"true,false\" name=\"Enable 5 Digit Addenda (UPC-A)\" id=\"Addenda5Digit\" gr=\"flag\">false</Key>",    // Disable UPCA 5 char addenda
            "",
            "<Key cmd=\"DEC_EAN13_ENABLED\" list=\"true,false\" name=\"EAN-13 Enable Symbology\" id=\"Enable\" gr=\"flag\">true</Key>",                                 // Enable EAN13
            "<Key cmd=\"DEC_EAN13_CHECK_DIGIT_TRANSMIT\" list=\"true,false\" name=\"EAN13 Check Digit Transmit\" id=\"Enable\" gr=\"true\">true</Key>",                 // Enable EAN13 check digit transmit
            "<Key cmd=\"DEC_EAN13_2CHAR_ADDENDA_ENABLED\" list=\"true,false\" name=\"Enable 2 Digit Addenda (EAN-13)\" id=\"Addenda2Digit\" gr=\"flag\">false</Key>",   // Disable EAN13 2 char addenda   
            "<Key cmd=\"DEC_EAN13_5CHAR_ADDENDA_ENABLED\" list=\"true,false\" name=\"Enable 5 Digit Addenda (EAN-13)\" id=\"Addenda5Digit\" gr=\"flag\">false</Key>",   // Disable EAN13 5 char addenda
            "</Section>"
        };

        // *** This is a sample profile section that can be used in the same way as the UPCA/EAN13 Addenda Enable/Disable sections to create a new profile. ***
        // I 2 of 5 Min/Max Length section.
        //  I2OF5_10_DIGIT_LABEL enables I 2 of 5 and sets the min and max label lengths to 10, so that only 10 digit labels can be scanned.
        string[] I2OF5MinMaxLengthSection = new string[]
        {
            "<Section flags=\"000\" name=\"I2OF5_10_DIGIT_LABEL\" id=\"I2OF5\">",                             // Profile name specfied in this line.
            "<Key cmd=\"DEVICE\" list=\"Internal,USB\" name=\"Device Type\">Internal</Key>",                  // Profile will apply to the Internal scanner.
            "<Key cmd=\"TYPE\"  list=\"Incremental,Full\" name=\"ProfileType\">Full</Key>",                   // Profile will set default configuration before applying profile.
            "<Key cmd=\"APPLY\" list=\"true,false\" min=\"\" name=\"ApplyProfileOnLoad\">false</Key>",        // POS application controls when profile is applied.
            "",
            "<Key cmd=\"DEC_I25_ENABLED\" list=\"true,false\" name=\"I2of5 Enable Symbology\" id=\"Enable\" gr=\"flag\">true</Key>",     // Enable I 2 of 5
            "<Key cmd=\"DEC_I25_MIN_LENGTH\" name=\"I2OF5 Minimum Character Length\" id=\"MinLength\" min=\"4\" max=\"80\">10</Key>",    // Set minimum label length to 10
            "<Key cmd=\"DEC_I25_MAX_LENGTH\" name=\"I2OF5 Maximum Character Length\" id=\"MaxLength\" min=\"4\" max=\"80\">10</Key>",    // Set maximum label length to 10
            "</Section>",
        };

        // Profile end.
        string[] profileEnd = new string[]
        {
            "</ConfigDoc>"
        };

        //        IReadOnlyList<uint> fullSymbologyList; // Uncomment if displaying all symbologies available with the scanner (see DisplaySupportedSymbologies()).

        private string currentName;
        private bool inNameElement = false;
        private bool inValueElement = false;

        public MainPage()
        {
            this.InitializeComponent();
            Current = this;
            App.scannerInfo si = new App.scannerInfo();
            si.ScannerName = "Default";
            cbScanners.Items.Add(si);
        }

        /// <summary>
        /// Event handler for received scanner data.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        public async void Scanner_DataReceived(object sender, BarcodeScannerDataReceivedEventArgs args)
        {
            try
            {
                // Get the data and symbology.
                string scanData = String.Empty;
                string symbology = BarcodeSymbologies.GetName(args.Report.ScanDataType);

                using (var datareader = Windows.Storage.Streams.DataReader.FromBuffer(args.Report.ScanDataLabel))
                {
                    scanData = datareader.ReadString(args.Report.ScanDataLabel.Length);
                }

                string length = scanData.Length.ToString();
                string result = scanData + " [" + length.ToString() + " chars, symbology=" + symbology + "]";

                await Dispatcher.RunAsync(
                    Windows.UI.Core.CoreDispatcherPriority.Normal,
                    () =>
                    {
                        tbScannedData.Text = result;
                        mdGoodReadBeep.AutoPlay = true;
                        mdGoodReadBeep.Play();
                        mdGoodReadBeep.AutoPlay = false;

                        triggering = false;
                        btnScan.Content = "Scan";
                    }
                );
            }
            catch (Exception ex)
            {
                MainPage.Current.NotifyUser("Exception thrown getting scanned data.");
                throw new Exception("Exception thrown getting scanned data: " + ex.Message);
            }
        }

         #region NavigationHelper registration

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e"></param>
        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            App.watcher.Stop();
            await Task.Delay(TimeSpan.FromMilliseconds(200));
            App.watcher.Added += DeviceAdded;
            App.watcher.Removed += DeviceRemoved;
            App.watcher.Start();
            await Task.Delay(TimeSpan.FromMilliseconds(600));
        }

        /// <summary>
        /// Invoked when this page is no longer displayed.
        /// </summary>
        /// <param name="e"></param>
        protected async override void OnNavigatedFrom(NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);
            App.watcher.Stop();
            await Task.Delay(TimeSpan.FromMilliseconds(200));
            App.watcher.Added -= DeviceAdded;
            App.watcher.Removed -= DeviceRemoved;
            App.watcher.Start();
            App.scannerData.scanManager.ResetTheScannerState();
        }
        #endregion

        /// <summary>
        /// Event handler that is called when a scanner is added.
        /// </summary>
        /// <param name="devDetector"></param>
        /// <param name="devInfo"></param>
        public async void DeviceAdded(DeviceWatcher devDetector, DeviceInformation devInfo)
        {
            await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
            {
                {
                    object sio = cbScanners.FindName(devInfo.Name);
                    if (sio == null)
                    {
                        // Save the scanner name and ID.
                        App.scannerInfo si = new App.scannerInfo();
                        si.ScannerName = devInfo.Name;
                        si.ScannerID = devInfo.Id;

                        // Add the scanner to the UI listbox.
                        cbScanners.Items.Add(si);
                    }
                }
            }
            );
        }

        /// <summary>
        /// Event handler that is called when a scanner is removed.
        /// </summary>
        /// <param name="devDetector"></param>
        /// <param name="devInfo"></param>
        public async void DeviceRemoved(DeviceWatcher devDetector, DeviceInformationUpdate devInfo)
        {
            await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
            {
                {
                    foreach (App.scannerInfo si in cbScanners.Items)
                    {
                        if (si.ScannerID == devInfo.Id)
                        {
                            // Remove the scanner from the UI listbox.
                            cbScanners.Items.Remove(si);

                            // If the scanner being removed is the current created/claimed scanner, clear 
                            // the other UI items affected by the scanner selection.
                            if( devInfo.Id == App.scannerData.scanManager.GetScannerID() )
                            {
                                lbSymbologies.Items.Clear();
                                cbProfiles.Items.Clear();
                                lbStatistics.Items.Clear();
                                tbScannedData.Text = "";
                            }
                            break;
                        }
                    }
                }
            }
            );
        }

        /// <summary>
        /// Display available and supported symbologies.
        /// </summary>
        public async void DisplaySupportedSymbologies()
        {
            BarcodeScanner currentScanner = App.scannerData.scanManager.GetBarcodeScanner();

            if( null != currentScanner )
            {
                // Go through the table of available symbologies (displayedSymbologiesList) and display those that are supported with the scanner.
                foreach (uint availableSymbology in displayedSymbologiesList)
                {
                    if (await currentScanner.IsSymbologySupportedAsync(availableSymbology))
                    {
                        lbSymbologies.Items.Add(BarcodeSymbologies.GetName(availableSymbology));
                    }
                }

                /* ******** Replace the code immediately above with the following code to display all symbologies supported by the current scanner.
                // Get all supported symbologies with this scanner and display them.
                fullSymbologyList = await currentScanner.GetSupportedSymbologiesAsync();
                foreach( uint symbology in fullSymbologyList )
                {
                    lbSymbologies.Items.Add(BarcodeSymbologies.GetName(symbology));
                }
                ********** */
            }
        }

        /// <summary>
        /// Display available and supported profiles.
        /// </summary>
        private void DisplayProfiles()
        {
            IReadOnlyList<string> ProfilesList = App.scannerData.scanManager.GetSupportedProfiles();
            foreach (string profile in ProfilesList)
            {
                cbProfiles.Items.Add(profile);
            }
        }

        public char[] GetChars(byte[] bytes)
        {
            char[] rChars = new char[bytes.Length / 2];
            for (int i = 0, j = 0; i < bytes.Length / 2; i++, j += 2)
            {
                rChars[i] = (char)(bytes[j] | bytes[j + 1] << 8);
            }
            return rChars;
        }

        /// <summary>
        /// This is the selection changed handler for the Scanners combo box. It initiates creation of the scanner selected.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void ComboBox_ScannerSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBox cb = (ComboBox)sender;

            if (null != cbScanners.SelectedItem)
            {
                try
                {
                    // Get the scanner selected in the UI and save its name.
                    App.scannerInfo si = (App.scannerInfo)cbScanners.SelectedItem;
                    string selectedScanner = si.ScannerName;
                    bool bCreateScannerStatus = false;

                    // If a claimed scanner already exists, remove it before creating the newly selected scanner.
                    if ( null != App.scannerData.scanManager.GetClaimedBarcodeScanner() )
                    {
                        await App.scannerData.scanManager.Dispose();
                    }

                    if ("Default" == selectedScanner)
                    {
                        // Create the default scanner.
                        bCreateScannerStatus = await App.scannerData.scanManager.CreateDefaultScannerObjectAsync();
                    }
                    else
                    {
                        // Create the scanner using its ID.
                        bCreateScannerStatus = await App.scannerData.scanManager.CreateScannerByIDObjectAsync(si.ScannerID);
                    }

                    // If the scanner was created, proceed with claiming and enabling it.
                    if ( (true == bCreateScannerStatus) && (null != App.scannerData.scanManager.GetBarcodeScanner()) )
                    {
                        // Claim the scanner
                        if (await App.scannerData.scanManager.ClaimBarcodeScannerAsync())
                        {
                            // Enable the Scanner
                            if (await App.scannerData.scanManager.EnableBarcodeScannerAsync())
                            {
                                // Make sure all symbologies are disabled and remove any previously displayed.
                                await App.scannerData.scanManager.DisableAllSymbologies();
                                lbSymbologies.Items.Clear();

                                // Display the symbologies available with this scanner.
                                DisplaySupportedSymbologies();

                                // Clear the scanned data text box.
                                tbScannedData.Text = "";

                                // Clear the Profiles UI combobox and the list of any previously activated profiles.
                                App.scannerData.scanManager.ClearActiveProfiles();
                                cbProfiles.Items.Clear();

                                // Display the profiles available with this scanner.
                                DisplayProfiles();

                                // Clear the statistics listbox.
                                lbStatistics.Items.Clear();
                                //                                    NotifyUser("Created/Claimed/Enabled scanner." + "\r\n" + "Device ID = " + App.scannerData.scanManager.GetDeviceId());
                            }
                        }
                    }
                    else
                    {
                        NotifyUser("Barcode scanner not created.");
                    }
                }
                catch (Exception ex)
                {
                    NotifyUser("Exception: Failed to create scanner: " + ex.Message);
                }
            }
        }

        /// <summary>
        /// This is the selection changed handler for the Settings Profile combo box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void ComboBox_ProfileSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Get the user selected profile.
            if( cbProfiles.SelectedItem != null )
            {
                SolidColorBrush Brush = new SolidColorBrush();
                Brush.Color = Windows.UI.Colors.DarkSlateBlue;
                this.cbProfiles.SelectedItem = Brush;

                try
                {
                    // Enable the selected profile.
                    await App.scannerData.scanManager.SetActiveProfile( cbProfiles.SelectedItem.ToString() );
                }
                catch (Exception ex)
                {
                    NotifyUser( "Exception: Failed to set profile: " + ex.Message );
                }

                // Clear any selected items in the Symbologies list box and any additional options checkboxes as they may no longer be enabled.
                if( null != lbSymbologies.SelectedItems )
                {
                    lbSymbologies.SelectedItems.Clear();
                }
            }
        }

        /// <summary>
        /// This is the selection changed handler for the Symbologies list box. It allows multiple selection of the
        /// symbologies in the list. As each symbology is selected, it will be enabled. Unselecting a symbology disables it.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void lbSymbologies_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            BarcodeScanner createdScanner = App.scannerData.scanManager.GetBarcodeScanner();
            ClaimedBarcodeScanner claimedScanner = App.scannerData.scanManager.GetClaimedBarcodeScanner();
            bool bSymbologyFound = false;

            List<uint> enableSymbologyList = new List<uint>();

            if (null != claimedScanner)
            {
                // For each symbology selected in the UI, find its match in the displayed symbologies table.
                foreach (object item in lbSymbologies.SelectedItems)
                {
//                    foreach (uint supportedSymbology in fullSymbologyList) // *** Replace the following line with this line if displaying all symbologies supported by the scanner.
                    foreach (uint supportedSymbology in displayedSymbologiesList)
                    {
                        if ((string)item == BarcodeSymbologies.GetName(supportedSymbology))
                        {
                            // Enable the selected symbology(s).
                            enableSymbologyList.Add( supportedSymbology );
                            await claimedScanner.SetActiveSymbologiesAsync( enableSymbologyList );

                            // Clear the Profiles combo box and any additional options checkboxes as any previously selected profiles and options may no longer be enabled.
                            if( (null != cbProfiles.SelectedItem) )
                            {
                                cbProfiles.SelectedIndex = -1;
                            }

                            // Save the list of enabled symbologies to restore after a suspend/resume.
                            App.scannerData.scanManager.SaveSymbologyList( enableSymbologyList );
                            bSymbologyFound = true;
                            break;
                        }
                    }

                    // Display message to user in the unlikely event the selected symbology isn't in the list
                    // of displayed (supported) symbologies.
                    if (!bSymbologyFound)
                    {
                        NotifyUser("Symbology " + (string)item + "not supported");
                    }
                }

                // If no symbologies selected, disable all of them.
                if (0 == lbSymbologies.SelectedItems.Count)
                {
                    lbSymbologies.SelectedItems.Clear();
                    await App.scannerData.scanManager.DisableAllSymbologies();
                }
            }
        }

        /// <summary>
        /// Click handler for the Create UPC/EAN Addenda Profiles button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void btnUPCEANAddons_Click(object sender, RoutedEventArgs e)
        {
            // Pressing this button creates profiles for enabling/disabling UPCA and EAN13 addenda.
            try
            {
                if (null != App.scannerData.scanManager.GetBarcodeScanner())
                {
                    // Open/create the subfolder where the profile is stored.
                    StorageFolder storageFolder = KnownFolders.DocumentsLibrary;
                    StorageFolder profilesFolder = await storageFolder.CreateFolderAsync("Profile", CreationCollisionOption.OpenIfExists);

                    // Open/create the profile file.
                    StorageFile profileFile = await profilesFolder.CreateFileAsync("HoneywellDecoderSettingsV2.Exm", CreationCollisionOption.ReplaceExisting);

                    // Write the profile header to the file.
                    await FileIO.WriteLinesAsync(profileFile, profileHeader);

                    // Write the UPCA/EAN13 Addenda section to the file.
                    await FileIO.AppendLinesAsync(profileFile, UPCAEAN13AddendaSection);

                    // Write the profile end to the file.
                    await FileIO.AppendLinesAsync(profileFile, profileEnd);

                    // Delay to allow time for new profile(s) to be processed.
                    await Task.Delay(50);

                    // Clear and redisplay the profiles combo box to pick up the newly created profile(s).
                    App.scannerData.scanManager.ClearActiveProfiles();
                    cbProfiles.Items.Clear();
                    DisplayProfiles();
                }
                else
                {
                    NotifyUser("You need to first select a scanner");
                }
            }
            catch (Exception ex)
            {
                NotifyUser("Exception while creating and applying profile: " + ex.Message);
            }
        }

        /// <summary>
        /// Click handler for the 'Scan' button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void ButtonScan_Click(object sender, RoutedEventArgs e)
        {
            Button btn = sender as Button;

            if (App.scannerData.scanManager.IsScannerEnabled())
            {
                try
                {
                    if (triggering)
                    {
                        btn.Content = "Scan";
                        await App.scannerData.scanManager.StopScan();
                        triggering = false;
                    }
                    else
                    {
                        btn.Content = "Stop scanning";
                        await App.scannerData.scanManager.StartScan();
                        triggering = true;
                    }
                }
                catch (Exception ex)
                {
                    NotifyUser("Exception in scanner trigger/untrigger: " + ex.Message);
                }
            }
        }

        /// <summary>
        /// Click handler for the Get Statistics button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void GetStatistics_Click(object sender, RoutedEventArgs e)
        {
            IBuffer statistics;

            if (App.scannerData.scanManager.GetClaimedBarcodeScanner() != null)
            {
                try
                {
                    string[] statCategories = { "" }; // Use this to obtain all available statistics.
                    // string[] statCategories = { "AccumulatedDecodeTime", "GoodScanCount", "DecodeSessionStarts" }; // Example of obtaining a subset of the statistics.
                    statistics = await App.scannerData.scanManager.RetrieveStatistics(statCategories);
                }
                catch (Exception ex)
                {
                    NotifyUser("Statistics retrieval failed. Error: " + ex.Message);
                    return;
                }

                DataReader reader = DataReader.FromBuffer(statistics);
                byte[] resultBytes = statistics.ToArray();
                char[] chars = GetChars(resultBytes);

                string result = new string(chars);
                byte[] utf8Bytes = Encoding.UTF8.GetBytes(chars);

                MemoryStream ms = new MemoryStream(utf8Bytes, 0, utf8Bytes.Length - 1);
                XmlReaderSettings xSet = new XmlReaderSettings();
                xSet.CheckCharacters = true;
                XmlReader xr = XmlReader.Create(ms, xSet);

                lbStatistics.Items.Clear();

                ProcessElementCallback DisplayStatisticsCallback = new ProcessElementCallback(DisplayStatistics);
                currentName = "";

                // Loop through top level of statistics xml elements
                while (ParseStatisticsXml(xr, DisplayStatisticsCallback)) ;
            }

            else
            {
                NotifyUser("Error: There is not a created/claimed scanner.");
            }
        }

        /// <summary>
        /// Click handler for the Reset Statistics button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ResetStatistics_Click(object sender, RoutedEventArgs e)
        {
            if (App.scannerData.scanManager.GetClaimedBarcodeScanner() != null)
            {
                App.scannerData.scanManager.ResetStatistics();
                lbStatistics.Items.Clear();
            }
            else
            {
                NotifyUser("Error: There is not a created/claimed scanner.");
            }
        }

        /// <summary>
        /// This function displays an error dialog. Clicking OK dismisses the dialog.
        /// </summary>
        /// <param name="strMessage"></param>
        public async void NotifyUser(string strMessage)
        {
            ContentDialog appErrorDialog = new ContentDialog()
            {
                Title = "Error",
                Content = strMessage,
                PrimaryButtonText = "Ok"
            };

            ContentDialogResult result = await appErrorDialog.ShowAsync();
        }

        /// <summary>
        /// This function displays in the statistics listbox each statistic and its value.
        /// </summary>
        /// <param name="kvp"></param>
        void DisplayStatistics(KeyValuePair<string, string> kvp)
        {
            lbStatistics.Items.Add(kvp.Key + "=" + kvp.Value);
        }

        /// <summary>
        /// This function parses the statistics xml returned by RetrieveStatistics.
        /// </summary>
        /// <param name="xr"></param>
        /// <param name="processElement"></param>
        /// <returns></returns>
        private bool ParseStatisticsXml(XmlReader xr, ProcessElementCallback processElement)
        {
            bool rVal = true;

            // Parse the statistics xml to display each statistic retrieved.
            try
            {
                if (xr.Read())
                {
                    switch (xr.NodeType)
                    {
                        case XmlNodeType.CDATA:
                            break;
                        case XmlNodeType.Comment:
                            break;
                        case XmlNodeType.DocumentType:
                            break;
                        case XmlNodeType.Element:
                            if (xr.Name == "Name")
                            {
                                inNameElement = true;
                            }
                            else if (xr.Name == "Value")
                            {
                                inValueElement = true;
                            }
                            else
                            {
                                currentName = xr.Name;
                            }
                            break;
                        case XmlNodeType.EndElement:
                            // Loop thru next level of statistics
                            while (ParseStatisticsXml(xr, processElement)) ;
                            break;
                        case XmlNodeType.EntityReference:
                            break;
                        case XmlNodeType.Entity:
                            break;
                        case XmlNodeType.ProcessingInstruction:
                            break;
                        case XmlNodeType.Text:
                            if (inNameElement)
                            {
                                currentName = xr.Value;
                                inNameElement = false;
                            }
                            else if (inValueElement)
                            {
                                KeyValuePair<string, string> kvp = new KeyValuePair<string, string>(currentName, xr.Value);
                                processElement(kvp);
                                inValueElement = false;
                                currentName = "";
                            }
                            else
                            {
                                KeyValuePair<string, string> kvp = new KeyValuePair<string, string>(currentName, xr.Value);
                                processElement(kvp);
                                currentName = "";
                            }
                            break;
                        case XmlNodeType.XmlDeclaration:
                            break;
                        default:
                            break;
                    }
                }
                else
                {
                    rVal = false;
                }
            }
            catch (Exception ex)
            {
                NotifyUser("Failed to parse statistics xml: " + ex.Message);
            }

            return rVal;
        }
    }
}