﻿
using System.Collections.Generic;
using System.Threading.Tasks;
using Windows.Devices.PointOfService;
using Windows.Storage.Streams;

namespace BarcodeScanningApp
{
    public interface IScanManager
    {

        /// <summary>
        /// Create scanner functions.
        /// </summary>
        /// <returns></returns>
        Task<bool> CreateDefaultScannerObjectAsync();
        Task<bool> CreateScannerByIDObjectAsync(string deviceId);

        /// <summary>
        /// Claim scanner functions.
        /// </summary>
        /// <returns></returns>
        Task<bool> ClaimBarcodeScannerAsync();
        ClaimedBarcodeScanner GetClaimedBarcodeScanner();

        /// <summary>
        /// Enable scanner functions.
        /// </summary>
        /// <returns></returns>
        Task<bool> EnableBarcodeScannerAsync();

        /// <summary>
        /// Additional scanner functions.
        /// </summary>
        /// <returns></returns>
        bool IsScannerEnabled();
        BarcodeScanner GetBarcodeScanner();
        Task RestoreScanner(string scannerId);
        void RemoveScanner(string scannerId);
        string GetDeviceId();
        bool CanScannerBeRestored(string scannerId);
        bool IsScannerInitialized();
        void ScannerInitializing(bool initialized);
        void ResetTheScannerState();
        string GetScannerID();
        Task Dispose();

        /// <summary>
        /// Symbology functions.
        /// </summary>
        /// <param name="symList"></param>
        void SaveSymbologyList(IReadOnlyList<uint> symList);
        Task DisableAllSymbologies();

        /// <summary>
        /// Software scan triggering functions.
        /// </summary>
        /// <returns></returns>
        Task StartScan();
        Task StopScan();

        /// <summary>
        /// Profile functions.
        /// </summary>
        /// <returns></returns>
        IReadOnlyList<string> GetSupportedProfiles();
        Task SetActiveProfile(string profileName);
        void ClearActiveProfiles();

        /// <summary>
        /// Statistics functions.
        /// </summary>
        /// <returns></returns>
        Task ResetStatistics();
        Task<IBuffer> RetrieveStatistics(string[] statCategories);
    }
}