﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using System.Threading.Tasks;
using Windows.UI.Core;
using Windows.UI.Popups;
using Honeywell.AIDC.CrossPlatform.Print;
using System.Threading;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace LabelPrinterSample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class PrintItemLabelPage : Page, IProgress<string>
    {
        // A pointer back to the main page.  This is needed if you want to call methods in MainPage such
        // as ShowErrorMessage
        MainPage rootPage = MainPage.Current;
        private static string itemName = "Honeywell AirGenius 4";
        private static string itemNumber = "92926003104";
        private PrintTask printTask = null;
        private System.Threading.SynchronizationContext mUIContext = SynchronizationContext.Current;

        public PrintItemLabelPage()
        {
            this.InitializeComponent();

            printTask = new PrintTask();
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            ItemNameTextBox.Text = itemName;
            ItemNumberTextBox.Text = itemNumber;
        }

        private void ItemNameTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(ItemNameTextBox.Text.Trim()))
            {
                itemName = string.Empty;
            }
            else
            {
                itemName = ItemNameTextBox.Text.Trim();
            }
        }

        private void ItemNumberTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(ItemNumberTextBox.Text.Trim()))
            {
                itemNumber = string.Empty;
            }
            else
            {
                itemNumber = ItemNumberTextBox.Text.Trim();
            }
        }

        private async void PrintButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if(string.IsNullOrEmpty(itemName) || string.IsNullOrEmpty(itemNumber))
                {
                    rootPage.ShowErrorMessage("ItemName and itemNumber are required.");
                }
                else
                {
                    this.PrintButton.IsEnabled = false;

                    //set printer lable data
                    Dictionary<string, string> varItemDictionary = new Dictionary<string, string>();
                    varItemDictionary.Add("ItemName", itemName);
                    varItemDictionary.Add("ItemNo", itemNumber);

                    InfoTextBlock.Text = "In printing process...";

                    await Task.Run(async () =>
                    {
                        string result = await printTask.DoPrintAsync(this,
                                                                    "Item",
                                                                    rootPage.printerId,
                                                                    rootPage.printerMacAddress,
                                                                    varItemDictionary);

                        await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                        {
                            InfoTextBlock.Text += Environment.NewLine + result;
                        });
                    });
                }
            }
            catch (PrinterException exp)
            {
                InfoTextBlock.Text += Environment.NewLine +
                                        "\nPrinter Exception:" +
                                        "\n- Message: " + exp.Message +
                                        "\n- Error code: 0x" + exp.HResult.ToString("X");
            }
            catch (Exception exp)
            {
                InfoTextBlock.Text += Environment.NewLine + 
                                      "\nException: " + 
                                      "\n- " + exp.Message +
                                      "\n- Error code: 0x" + exp.HResult.ToString("X");
            }
            finally
            {
                this.PrintButton.IsEnabled = true;
            }
        }

        public void Report(string statusText)
        {
            mUIContext.Post(_ => {
                InfoTextBlock.Text += Environment.NewLine + statusText;
            }
            , null);
        }
    }
}


