﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Honeywell.AIDC.CrossPlatform.Print;
using Windows.Storage;

namespace LabelPrinterSample
{
    public class PrintTask
    {
        private LabelPrinter labelPrinter = null;
        private string printerURI = string.Empty;
        public static string profiles = null;

        public async Task<String> DoPrintAsync(IProgress<string> updateUIProgress,
                                                string printType,
                                                string printerId,
                                                string printerAddress,
                                                Dictionary<string, string> varDictionary)
        {
            string result = string.Empty;

            try
            {
                if (string.IsNullOrEmpty(printerId))
                {
                    return "Printer Id is not correct.";
                }

                if(!GetFormatPrinterURI(printerAddress))
                {
                    return "Printer MAC address is not correct.";
                }

                // Read printer command profile into a string
                if (null == profiles)
                {
                    profiles = await readFileFromAssetsFolder(updateUIProgress);
                }

                // Open printer connection
                labelPrinter = new LabelPrinter(profiles, printerId, printerURI);


                updateUIProgress.Report("- connects printer");
                labelPrinter.Connect();

                if ("Item" == printType)
                {
                    updateUIProgress.Report("- writes item label");
                    labelPrinter.WriteLabel("ItemLabel", varDictionary);
                }
                else if ("URL" == printType)
                {
                    updateUIProgress.Report("- writes URL label");
                    labelPrinter.WriteLabel("URL_QRLabel", varDictionary);
                }

                result = "Printed Successfully.";
            }
            finally
            {
                if (null != labelPrinter)
                {
                    labelPrinter.Disconnect();
                    labelPrinter.Dispose();
                }
            }

            return result;
        }

        private async Task<string> readFileFromAssetsFolder(IProgress<string> updateUIProgress)
        {
            updateUIProgress.Report("- read printer_profiles.json file");

            string fileContent = null;

            var file = await StorageFile.GetFileFromApplicationUriAsync(new Uri("ms-appx:///Assets/printer_profiles.json"));
            if (file != null)
            {
                fileContent = await FileIO.ReadTextAsync(file);
                if (fileContent == null)
                    updateUIProgress.Report("- Could not read profile conten");
            }
            else
            {
                updateUIProgress.Report("- Could not access printer_profiles.json file");
            }

            return fileContent;
        }

        private bool GetFormatPrinterURI(string address)
        {
            bool status = false;
            //If the MAC address only contains hex digits without the
            //":" delimiter, insert ":" to the MAC address string
            if (12 == address.Length && false == address.Contains(":"))
            {
                for (int i = 2; i < 17;)
                {
                    address = address.Insert(i, ":");
                    i = i + 3;
                }

                printerURI = "bt://" + address;
                status = true;
            }
            else if (17 == address.Length)
            {
                //check if address format is correct
                for (int i = 2; i < 17;)
                {
                    if (address[i] != ':')
                    {
                        return false;
                    }
                    i = i + 3;
                }
                printerURI = "bt://" + address;
                status = true;
            }
            else
            {
                status = false;
            }

            return status;
        }
    }
}
