﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Core;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Honeywell.AIDC.CrossPlatform.Print;
using System.Threading;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace LabelPrinterSample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class PrintURLLabelPage : Page, IProgress<string>
    {
        // A pointer back to the main page.  This is needed if you want to call methods in MainPage such
        // as ShowErrorMessage
        MainPage rootPage = MainPage.Current;
        private string urlAddress = "http://www.honeywellaidc.com";
        private string line1Text = "Scan for more info";
        private string line2Text = "www.honeywellaidc.com";
        private PrintTask printTask = null;
        private System.Threading.SynchronizationContext mUIContext = SynchronizationContext.Current;

        public PrintURLLabelPage()
        {
            this.InitializeComponent();

            printTask = new PrintTask();
        } 

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            rootPage = MainPage.Current;

            //display stored URL address, Line1 text, and line2 text
            URLTextBox.Text = urlAddress;
            Line1TextBox.Text = line1Text;
            Line2TextBox.Text = line2Text;
        }

        private void URLTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(URLTextBox.Text.Trim()))
            {
                urlAddress = string.Empty;
            }
            else
            {
                urlAddress = URLTextBox.Text.Trim();
            }
        }

        private void Line1TextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(Line1TextBox.Text.Trim()))
            {
                line1Text = string.Empty;
            }
            else
            {
                line1Text = Line1TextBox.Text.Trim();
            }
        }

        private void Line2TextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(Line2TextBox.Text.Trim()))
            {
                line2Text = string.Empty;
            }
            else
            {
                line2Text = Line2TextBox.Text.Trim();
            }
        }

        private async void PrintButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(urlAddress) ||
                    string.IsNullOrEmpty(line1Text) ||
                    string.IsNullOrEmpty(line2Text))
                {
                    rootPage.ShowErrorMessage("URL address, Line1Text, and Line2Text are correctrequired.");
                }
                else
                {
                    this.PrintButton.IsEnabled = false;

                    //set URL lable printing data
                    Dictionary<string, string> varUrlDictionary = new Dictionary<string, string>();
                    varUrlDictionary.Add("URL", urlAddress);
                    varUrlDictionary.Add("TextLine1", line1Text);
                    varUrlDictionary.Add("TextLine2", line2Text);

                    InfoTextBlock.Text = "In printing process...";

                    await Task.Run(async () =>
                    {
                        string result = await printTask.DoPrintAsync(this,
                                                                    "URL",
                                                                    rootPage.printerId,
                                                                    rootPage.printerMacAddress,
                                                                    varUrlDictionary);

                        await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                        {
                            InfoTextBlock.Text += Environment.NewLine + result;
                        });
                    });
                }
            }
            catch (PrinterException exp)
            {
                InfoTextBlock.Text += Environment.NewLine +
                                        "\nPrinter Exception:" +
                                        "\n- Message: " + exp.Message +
                                        "\n- Error code: 0x" + exp.HResult.ToString("X");
            }
            catch (Exception exp)
            {
                InfoTextBlock.Text += Environment.NewLine +
                                      "\nException: " +
                                      "\n- " + exp.Message +
                                      "\n- Error code: 0x" + exp.HResult.ToString("X");
            }
            finally
            {
                this.PrintButton.IsEnabled = true;
            }
        }

        public void Report(string statusText)
        {
            mUIContext.Post(_ => {
                InfoTextBlock.Text += Environment.NewLine + statusText;
            }
            , null);
        }
    }
}
