﻿using Honeywell.AIDC.CrossPlatform.Print;
using LinePrinterShared;
using System;
using System.Threading;
using System.Threading.Tasks;
using Windows.UI.Core;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;


namespace LinePrinterUW
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page, IProgress<string>
    {
        PrintTask printTask = null;
        MessageDialog messageDialog = null;
        private System.Threading.SynchronizationContext mUIContext = SynchronizationContext.Current;

        public MainPage()
        {
            this.InitializeComponent();

            printTask = new PrintTask();
        }

        //set and get print graphic status
        public bool IsPrintGraphic { get; set; }

        private async void printBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                printBtn.IsEnabled = false;
                InfoTextBlock.Text = string.Empty;

                string printerID = this.printerName.Text.Trim();
                string printerURI = "bt://" + this.printerAddress.Text.Trim();

                if (String.IsNullOrWhiteSpace(this.printerName.Text.Trim()))
                {
                    this.showError("Printer Name is required");
                    this.printerName.Focus(FocusState.Programmatic);
                }
                else if (String.IsNullOrWhiteSpace(this.printerAddress.Text.Trim()))
                {
                    this.showError("Please enter printer's address");
                    this.printerAddress.Focus(FocusState.Programmatic);
                }
                else
                {
                    IsPrintGraphic = printGraphicCheckBox.IsChecked.Value;
                    InfoTextBlock.Text = "In printing process...";

                    await Task.Run(async () =>
                    {
                        bool success = await printTask.DoPrintAsync(this, printerID, printerURI, IsPrintGraphic);

                        await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                        {
                            if (success)
                                InfoTextBlock.Text += "\nPrinted Successfully.";
                            else
                                InfoTextBlock.Text += "\nFailed on printing.";
                        });
                    });
                }
            }
            catch (PrinterException exp)
            {
                InfoTextBlock.Text += Environment.NewLine +
                                        "\nPrinter Exception:" +
                                        "\n- Message: " + exp.Message +
                                        "\n- Error code: 0x" + exp.HResult.ToString("X");
            }
            catch (Exception exp)
            {
                InfoTextBlock.Text += Environment.NewLine +
                                      "\nException: " +
                                      "\n- " + exp.Message +
                                      "\n- Error code: 0x" + exp.HResult.ToString("X");
            }
            finally
            {
                printBtn.IsEnabled = true;
            }
        }

        public void Report(string statusText)
        {
            mUIContext.Post(_ => {
                InfoTextBlock.Text += Environment.NewLine + statusText;
            }
            , null);
        }

        // change the printer address to nn:nn:nn:nn:nn:nn format
        private void printerAddress_LostFocus(object sender, RoutedEventArgs e)
        {
            string printerAddress = this.printerAddress.Text.Trim();
            string macAddress = null;

            if (!String.IsNullOrWhiteSpace(printerAddress) && false == printerAddress.Contains(":") && 12 == printerAddress.Length)
            {
                // If the MAC address only contains hex digits without the
                // ":" delimiter, then add ":" to the MAC address string.
                char[] cAddr = new char[17];

                for (int i = 0, j = 0; i < 12; i += 2)
                {
                    printerAddress.CopyTo(i, cAddr, j, 2);
                    j += 2;
                    if (j < 17)
                    {
                        cAddr[j++] = ':';
                    }
                }
                macAddress = new String(cAddr);
            }

            if (!string.IsNullOrEmpty(macAddress))
            {
                this.printerAddress.Text = macAddress;
            }
        }

        // display error-alert popup
        private async void showError(string errorString)
        {
            messageDialog = new MessageDialog(errorString, "Error!");
            await messageDialog.ShowAsync();
        }
    }
}
